package org.artifactory.ph;

import org.artifactory.common.repo.PackageType;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.service.PackageSpecificationAdapter;
import org.artifactory.descriptor.repo.RepoType;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.openMocks;

@Test
public class PackageSpecificationImplTest {

    private PackageSpecificationImpl packageIndexChecker;

    @Mock
    private PackageSpecificationAdapter packageSpecificationAdapter;

    @BeforeMethod
    private void setup() {
        openMocks(this);
        packageIndexChecker = new PackageSpecificationImpl(packageSpecificationAdapter);
    }

    @Test
    public void testIsExpirable() throws PackageException {
        when(packageSpecificationAdapter.isExpirable(any(), any())).thenReturn(false);
        when(packageSpecificationAdapter.isSupportedPackage(any(PackageType.class))).thenReturn(true);

        Assert.assertFalse(packageIndexChecker.isExpirable(RepoType.CRAN, "repoKey", "a/b/c"));
        verify(packageSpecificationAdapter, times(1)).isExpirable(eq(PackageType.CRAN), eq("a/b/c"));
        Mockito.reset(packageSpecificationAdapter);
        when(packageSpecificationAdapter.isExpirable(any(), any())).thenReturn(true);
        when(packageSpecificationAdapter.isSupportedPackage(any(PackageType.class))).thenReturn(true);
        Assert.assertTrue(packageIndexChecker.isExpirable(RepoType.CRAN, "repoKey", "a/b/c"));
        verify(packageSpecificationAdapter, times(1)).isExpirable(eq(PackageType.CRAN), eq("a/b/c"));
    }

    @Test
    public void testIsLocalGenerated() throws PackageException {
        when(packageSpecificationAdapter.isLocalGenerated(any(), any())).thenReturn(false);
        when(packageSpecificationAdapter.isSupportedPackage(any(PackageType.class))).thenReturn(true);
        Assert.assertFalse(packageIndexChecker.isLocalGenerated(RepoType.CRAN, "repoKey", "a/b/c"));
        verify(packageSpecificationAdapter, times(1)).isLocalGenerated(eq(PackageType.CRAN), eq("a/b/c"));
        Mockito.reset(packageSpecificationAdapter);
        when(packageSpecificationAdapter.isLocalGenerated(any(), any())).thenReturn(true);
        when(packageSpecificationAdapter.isSupportedPackage(any(PackageType.class))).thenReturn(true);
        Assert.assertTrue(packageIndexChecker.isLocalGenerated(RepoType.CRAN, "repoKey", "a/b/c"));
        verify(packageSpecificationAdapter, times(1)).isLocalGenerated(eq(PackageType.CRAN), eq("a/b/c"));
    }

    @Test
    public void testShouldExcludePropertyFromMirroring() throws PackageException {
        when(packageSpecificationAdapter.isLocallyGeneratedProperty(PackageType.CRAN, "repoKey")).thenReturn(false);
        when(packageSpecificationAdapter.isSupportedPackage(any(PackageType.class))).thenReturn(true);
        Assert.assertFalse(packageIndexChecker.shouldExcludePropertyFromMirroring(RepoType.CRAN, "repoKey"));
        verify(packageSpecificationAdapter, times(1)).isLocallyGeneratedProperty(eq(PackageType.CRAN), eq("repoKey"));
        Mockito.reset(packageSpecificationAdapter);
        when(packageSpecificationAdapter.isLocallyGeneratedProperty(PackageType.CRAN, "repoKey")).thenReturn(true);
        when(packageSpecificationAdapter.isSupportedPackage(any(PackageType.class))).thenReturn(true);
        Assert.assertTrue(packageIndexChecker.shouldExcludePropertyFromMirroring(RepoType.CRAN, "repoKey"));
        verify(packageSpecificationAdapter, times(1)).isLocallyGeneratedProperty(eq(PackageType.CRAN), eq("repoKey"));
    }

    @Test
    public void testIsPackageArtifact() throws PackageException {
        when(packageSpecificationAdapter.isSupportedPackage(any(String.class))).thenReturn(true);
        Assert.assertFalse(packageIndexChecker.isPackageArtifact("repoKey", "path"));
        verify(packageSpecificationAdapter, times(1)).isPackageArtifact(eq("repoKey"), eq("path"));
        Mockito.reset(packageSpecificationAdapter);

        when(packageSpecificationAdapter.isSupportedPackage(any(String.class))).thenReturn(true);
        when(packageSpecificationAdapter.isPackageArtifact("repoKey", "path")).thenReturn(true);
        Assert.assertTrue(packageIndexChecker.isPackageArtifact("repoKey", "path"));
        verify(packageSpecificationAdapter, times(1)).isPackageArtifact(eq("repoKey"), eq("path"));
    }
}