package org.artifactory.addon.observability;

import org.artifactory.addon.Addon;
import org.artifactory.api.request.ArtifactoryResponse;
import org.artifactory.repo.RepoPath;
import org.artifactory.util.UnsupportedByLicenseException;
import org.glassfish.jersey.media.multipart.FormDataMultiPart;

import javax.ws.rs.core.Response;
import java.io.IOException;

/**
 * This addon is used to handle observability system repos related requests
 *
 * @author Tomer Slutsky
 */
public interface ObservabilityAddon extends Addon {
    UnsupportedByLicenseException OBSERVABILITY_UNSUPPORTED_OPERATION_EXCEPTION = new UnsupportedByLicenseException(
            "Observability is only available on Pro+ licensed Artifactory instances.");

    /**
     * Adds the log to an observability repo
     */
    default Response addLogToRepo(RepoPath repoPath, ObservabilityInfo info) throws IOException {
        throw OBSERVABILITY_UNSUPPORTED_OPERATION_EXCEPTION;
    }

    /**
     * Convert the received formParams to observabilityInfo
     */
    default ObservabilityInfo toObservabilityInfo(FormDataMultiPart formDataMultiPart)
            throws IncompleteFormDataException {
        throw OBSERVABILITY_UNSUPPORTED_OPERATION_EXCEPTION;
    }

    /**
     * Gets the logs and their metadata from the New folder, and the number of successful/failed logs
     */
    default Response getLogInfo(RepoPath newPath, RepoPath failurePath, RepoPath successPath) {
        throw OBSERVABILITY_UNSUPPORTED_OPERATION_EXCEPTION;
    }

    /**
     * Download a requested logFile based on it's path
     */
    default void downloadLogFile(RepoPath repoPath, ArtifactoryResponse res) throws IOException {
        throw OBSERVABILITY_UNSUPPORTED_OPERATION_EXCEPTION;
    }

    /**
     * Move a log file from New folder to success/failure folder
     */
    default void moveLogFileToNewStatusFolder(RepoPath pathTo, RepoPath pathFrom) {
        throw OBSERVABILITY_UNSUPPORTED_OPERATION_EXCEPTION;
    }

    /**
     * Delete a logfile from requested path
     */
    default void deleteLogFile(RepoPath path) {
        throw OBSERVABILITY_UNSUPPORTED_OPERATION_EXCEPTION;
    }

}
