package org.artifactory.util;

import com.thoughtworks.xstream.XStream;
import org.artifactory.exception.DocumentBuildException;
import org.jdom2.Document;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.LinkedList;

/**
 * @author Volodymyr Shulga
 * @author Shahaf Golan
 */
class XmlUtilsTest {

    @Test
    void testAllowedType(){
        XStream xStream = new XStream();
        XmlUtils.hardenXStream(xStream);
        Assertions.assertDoesNotThrow(()->xStream.fromXML(xStream.toXML(new HashMap())));
        Assertions.assertDoesNotThrow(()->xStream.fromXML(xStream.toXML(new ArrayList())));
        Assertions.assertDoesNotThrow(()->xStream.fromXML(xStream.toXML(new LinkedList())));
        Assertions.assertDoesNotThrow(()->xStream.fromXML(xStream.toXML(new LinkedHashSet())));
        Assertions.assertDoesNotThrow(()->xStream.fromXML(xStream.toXML(new HashSet())));
        Assertions.assertDoesNotThrow(()->xStream.fromXML(xStream.toXML(new Object[1])));
        Assertions.assertDoesNotThrow(()->xStream.fromXML(xStream.toXML("test")));
    }

    @Test
    void shouldParseSuccessfully() {
        Document document = XmlUtils
                .parse(XmlUtilsTest.class.getResourceAsStream("/config/test/config.2.2.3.with.docker.repos.xml"));
        Assertions.assertNotNull(document);
    }

    @Test
    void shouldFailParseForDoctype() throws IOException {

        String config = Files.readString(
                Path.of(
                        XmlUtilsTest.class.getResource("/config/test/config.3.0.2_doctype.xml").getPath()
                )
        );
        Assertions.assertNotNull(config);
        Assertions.assertThrows(DocumentBuildException.class, () -> {
            XmlUtils.parseStrict(config);
        }, "Error on line 2: DOCTYPE is disallowed when the feature " +
                "\\\"http://apache.org/xml/features/disallow-doctype-decl\\\" set to true.");
    }

    @Test
    void shouldProcessDoctypeWithoutStrictOption() {
        Document document = XmlUtils.parse(
                XmlUtilsTest.class.getResourceAsStream("/config/test/config.3.0.2_doctype.xml")
        );
        Assertions.assertNotNull(document);
    }
}
