package org.artifactory.core.api.ph.service;

import org.artifactory.core.api.ph.model.PackageArtifact;

import java.util.Map;
import java.util.stream.Stream;

public interface PackageSearchArtifactService {

    /**
     * Find artifacts by path and name (only name can include one '*' to use a pattern search)
     * Only artifacts with read permissions are returned
     *
     * @param path The path of the artifact (without leading '/', e.g. root is {@code ""})
     * @param name The name of the artifact. Can include one '*'.
     * @return an {@link Stream} with the {@link PackageArtifact}s that match the searched path and name
     */
    Stream<PackageArtifact> findArtifactsInExactPath(String repoKey, String path, String name);

    /**
     * Get the children artifacts of the given directory path.
     * Only artifacts with read permissions are returned
     *
     * @param path     The path for which to get the children (without leading '/', e.g. root is {@code ""})
     *                 The path can include one '*' at the end only.
     * @param name     The name of the artifact. Can include one '*'.
     * @param minDepth The minimum file depth to search from.
     * @return an {@link Stream} with the {@link PackageArtifact}s that match the searched path and name
     */
    Stream<PackageArtifact> findChildrenArtifacts(String repoKey, String path, String name, int minDepth);

    /**
     * Find artifacts by matching properties.
     * An artifact is returned only if it has properties that match all given search properties.
     * Only artifacts with read permissions are returned
     *
     * @param properties the properties to match. The value can include one '*' at the end to use a pattern match.
     * @param path       an optional folder path to search in. If not empty, it will search in the entire repository.
     *                   The path can include one '*' at the end only.
     * @return an {@link Stream} with the {@link PackageArtifact}s that match all searched properties
     */
    Stream<PackageArtifact> findArtifactsByProperties(String repoKey, String path, Map<String, String> properties);

    Stream<PackageArtifact> findArtifactsByPropertiesAndPaging(String repoKey, String path,
            Map<String, String> properties, int offset, int limit);

}
