package org.artifactory.core.api.ph.service;

import org.artifactory.common.resource.BaseResource;
import org.artifactory.core.api.ph.exception.PackageException;

import java.io.InputStream;

/**
 * API that Artifactory provides for the package handler layer for deploy/upload artifacts
 */
public interface PackageUploadService {

    /**
     * Upload an artifact to local or leading local in virtual repository
     *
     * @param repoKey     Repository key
     * @param path        Artifact path
     * @param inputStream Input stream
     * @throws PackageException in case od missing permission missing repository or internal error
     */
    void upload(String repoKey, String path, InputStream inputStream) throws PackageException;

    /**
     * This is INTERNAL method, it should be used only in the package handler framework and should not be exposed
     * to the various package modules
     *
     * @param repoKey     the virtual repo to upload
     * @param path        the path to upload
     * @param inputStream the artifact content
     * @return file resource
     * @throws PackageException in case of repository not found or any internal error
     */
    BaseResource uploadToVirtualCache(String repoKey, String path, InputStream inputStream) throws
            PackageException;
}
